@@ -1,13 +1,14 @@
-from select import select
 from typing import Any, Dict, List, Optional
 
 import pandas as pd
 from datapipe.compute import (Catalog, ComputeStep, DataStore, Pipeline,
                               run_steps, run_steps_changelist)
+from datapipe.store.database import TableStoreDB
 from datapipe.types import ChangeList
-from fastapi import FastAPI
+from fastapi import FastAPI, Response
 from pydantic import BaseModel, Field
 from sqlalchemy.sql.expression import select
+from sqlalchemy.sql.functions import count
 
 
 class PipelineStepResponse(BaseModel):
@@ -122,10 +123,9 @@ def DatpipeAPIv1(ds: DataStore, catalog: Catalog, pipeline: Pipeline, steps: Lis
         meta_schema = dt.meta_table.sql_schema
         meta_tbl = dt.meta_table.sql_table
 
-        sql = select(*meta_schema).\
-            where(meta_tbl.c.delete_ts.is_(None)).\
-            offset(page * page_size).\
-            limit(page_size)
+        sql = select(*meta_schema)
+        sql = sql.where(meta_tbl.c.delete_ts.is_(None))
+        sql = sql.offset(page * page_size).limit(page_size)
 
         meta_df = pd.read_sql_query(
             sql,
@@ -148,26 +148,28 @@ def DatpipeAPIv1(ds: DataStore, catalog: Catalog, pipeline: Pipeline, steps: Lis
     def get_data_post(req: GetDataRequest) -> GetDataResponse:
         dt = catalog.get_datatable(ds, req.table)
 
-        meta_schema = dt.meta_table.sql_schema
-        meta_tbl = dt.meta_table.sql_table
+        assert(isinstance(dt.table_store, TableStoreDB))
 
-        sql = select(*meta_schema).\
-            where(meta_tbl.c.delete_ts.is_(None)).\
-            offset(req.page * req.page_size).\
-            limit(req.page_size)
+        sql_schema = dt.table_store.data_sql_schema
+        sql_table = dt.table_store.data_table
+
+        sql = select(*sql_schema).select_from(sql_table)
+        # Data table has no delete_ts
+        # sql = sql.where(sql_table.c.delete_ts.is_(None))
+        sql = sql.offset(req.page * req.page_size).limit(req.page_size)
 
         for col, val in req.filters.items():
-            sql = sql.where(meta_tbl.c[col] == val)
+            sql = sql.where(sql_table.c[col] == val)
 
-        print(sql.compile(compile_kwargs={"literal_binds": True}))
+        sql_count = select(count()).select_from(sql_table)
+        for col, val in req.filters.items():
+            sql_count = sql_count.where(sql_table.c[col] == val)
 
         meta_df = pd.read_sql_query(
             sql,
             con=ds.meta_dbconn.con,
         )
 
-        print(meta_df)
-
         if not meta_df.empty:
             data_df = dt.get_data(meta_df)
         else:
@@ -176,7 +178,7 @@ def DatpipeAPIv1(ds: DataStore, catalog: Catalog, pipeline: Pipeline, steps: Lis
         return GetDataResponse(
             page=req.page,
             page_size=req.page_size,
-            total=len(meta_df),
+            total=dt.table_store.dbconn.con.execute(sql_count).fetchone()[0],
             data=data_df.to_dict(orient="records"),
         )
 
@@ -235,4 +237,14 @@ def DatpipeAPIv1(ds: DataStore, catalog: Catalog, pipeline: Pipeline, steps: Lis
     def run():
         run_steps(ds=ds, steps=steps)
 
+    @app.get("/get-file")
+    def get_file(filepath: str):
+        import mimetypes
+
+        import fsspec
+
+        with fsspec.open(filepath) as f:
+            mime = mimetypes.guess_type(filepath)
+            return Response(content=f.read(), media_type=mime[0])
+
     return app
